@@ -6,7 +6,7 @@ use 5.008001;
 use Carp qw( croak );
 
 # ABSTRACT: Write Perl bindings to non-Perl libraries with FFI. No XS required.
-our $VERSION = '0.20'; # VERSION
+our $VERSION = '0.21'; # VERSION
 
 # Platypus Man,
 # Platypus Man,
@@ -141,7 +141,8 @@ sub type
 
   croak "alias conflicts with existing type" if defined $alias && (defined $type_map->{$alias} || defined $self->{types}->{$alias});
 
-  if($name =~ /-\>/ || $name =~ /^record\s*\([0-9A-Z:a-z_]+\)$/)
+  if($name =~ /-\>/ || $name =~ /^record\s*\([0-9A-Z:a-z_]+\)$/
+  || $name =~ /^string(_rw|_ro|\s+rw|\s+ro|\s*\([0-9]+\))$/)
   {
     # for closure and record types we do not try to convet into the
     # basic type so you can have many many many copies of a given
@@ -345,6 +346,34 @@ sub sizeof
 }
 
 
+sub alignof
+{
+  my($self, $name) = @_;
+  my $meta = $self->type_meta($name);
+  
+  croak "cannot determine alignment of record"
+    if $meta->{type} eq 'record';
+  
+  require FFI::Platypus::ConfigData;
+
+  my $ffi_type;
+  if($meta->{type} eq 'pointer')
+  {
+    $ffi_type = 'pointer';
+  }
+  elsif($meta->{type} eq 'string' && $meta->{fixed_size})
+  {
+    $ffi_type = 'uint8';
+  }
+  else
+  {
+    $ffi_type = $meta->{ffi_type};
+  }
+  
+  FFI::Platypus::ConfigData->config('align')->{$ffi_type};
+}
+
+
 sub find_lib
 {
   my $self = shift;
@@ -447,7 +476,7 @@ sub DESTROY
 
 package FFI::Platypus::Function;
 
-our $VERSION = '0.20'; # VERSION
+our $VERSION = '0.21'; # VERSION
 
 use overload '&{}' => sub {
   my $ffi = shift;
@@ -459,7 +488,7 @@ package FFI::Platypus::Closure;
 use Scalar::Util qw( refaddr);
 use Carp qw( croak );
 
-our $VERSION = '0.20'; # VERSION
+our $VERSION = '0.21'; # VERSION
 
 our %cbdata;
 
@@ -486,13 +515,13 @@ sub DESTROY
 
 package FFI::Platypus::ClosureData;
 
-our $VERSION = '0.20'; # VERSION
+our $VERSION = '0.21'; # VERSION
 
 package FFI::Platypus::Type;
 
 use Carp qw( croak );
 
-our $VERSION = '0.20'; # VERSION
+our $VERSION = '0.21'; # VERSION
 
 sub new
 {
@@ -511,13 +540,17 @@ sub new
   
   my $ffi_type;
   my $platypus_type;
-  my $array_or_record_size = 0;
+  my $size = 0;
   my $classname;
-  
-  if($type eq 'string')
+  my $rw = 0;
+
+  if($type =~ /^string(_rw|_ro|\s+ro|\s+rw|\s*\([0-9]+\)|)$/)
   {
+    my $extra = $1;
     $ffi_type = 'pointer';
     $platypus_type = 'string';
+    $rw = 1 if $extra =~ /rw$/;
+    $size = $1 if $extra =~ /\(([0-9]+)\)$/;
   }
   elsif($type =~ /^record\s*\(([0-9:A-Za-z_]+)\)$/)
   {
@@ -525,7 +558,7 @@ sub new
     $platypus_type = 'record';
     if($1 =~ /^([0-9]+)$/)
     {
-      $array_or_record_size = $1;
+      $size = $1;
     }
     else
     {
@@ -537,11 +570,11 @@ sub new
       }
       if($classname->can('ffi_record_size'))
       {
-        $array_or_record_size = $classname->ffi_record_size;
+        $size = $classname->ffi_record_size;
       }
       elsif($classname->can('_ffi_record_size'))
       {
-        $array_or_record_size = $classname->_ffi_record_size;
+        $size = $classname->_ffi_record_size;
       }
       else
       {
@@ -557,7 +590,7 @@ sub new
   {
     $ffi_type = $type;
     $platypus_type = 'array';
-    $array_or_record_size = $1;
+    $size = $1;
   }
   else
   {
@@ -565,7 +598,7 @@ sub new
     $platypus_type = 'ffi';
   }
   
-  $class->_new($ffi_type, $platypus_type, $array_or_record_size, $classname);
+  $class->_new($ffi_type, $platypus_type, $size, $classname, $rw);
 }
 
 1;
@@ -582,7 +615,7 @@ FFI::Platypus - Write Perl bindings to non-Perl libraries with FFI. No XS requir
 
 =head1 VERSION
 
-version 0.20
+version 0.21
 
 =head1 SYNOPSIS
 
@@ -600,12 +633,15 @@ version 0.20
 
 =head1 DESCRIPTION
 
-Platypus provides a method for creating interfaces to machine code 
-libraries.  This implementation uses C<libffi>, a library that provides 
-Foreign Function Interfaces for a number of other languages, including 
-Ruby and Python.  Platypus can be used in stand alone scripts, or to 
-create Perl extensions for CPAN.  There are a number of reasons why you 
-might want to write extensions with FFI instead of XS:
+Platypus is an library for creating interfaces to machine code libraries 
+written in languages like C, L<C++|FFI::Platypus::Lang::CPP>, Fortran, 
+L<Rust|FFI::Platypus::Lang::Rust>, 
+L<Pascal|FFI::Platypus::Lang::Pascal>. Essentially anything that gets 
+compiled into machine code.  This implementation uses C<libffi> to 
+accomplish this task.  C<libffi> is battle tested by a number of other 
+scripting and virtual machine languages, such as Python and Ruby to 
+serve a similar role.  There are a number of reasons why you might want 
+to write an extension with Platypus instead of XS:
 
 =over 4
 
@@ -628,6 +664,14 @@ be shared between different languages.
 One of those "other" languages could be Perl 6 and Perl 6 already has an 
 FFI interface I am told.
 
+=item FFI / Platypus can be reimplemented
+
+In a bright future with multiple implementations of Perl 5, each 
+interpreter will have its own implementation of Platypus, allowing 
+extensions to be written once and used on multiple platforms, in much 
+the same way that Ruby-FFI extensions can be use in Ruby, JRuby and 
+Rubinius.
+
 =item FFI / Platypus is pure perl (sorta)
 
 One Platypus script or module works on any platform where the libraries 
@@ -639,11 +683,13 @@ platform specific Perl library path.
 =item FFI / Platypus is not C or C++ centric
 
 XS is implemented primarily as a bunch of C macros, which requires at 
-least some understanding of C, (sadly) the C pre-processor, and some C++ 
-caveats (since on some platforms Perl is compiled and linked with a C++ 
+least some understanding of C, the C pre-processor, and some C++ caveats 
+(since on some platforms Perl is compiled and linked with a C++ 
 compiler). Platypus on the other hand could be used to call other 
-compiled languages, like Fortran, Rust, Go or even assembly, allowing 
-you to focus on your strengths.
+compiled languages, like Fortran, L<Rust|FFI::Platypus::Lang::Rust>, 
+L<Pascal|FFI::Platypus::Lang::Pascal>, L<C++|FFI::Platypus::Lang::CPP>, 
+Go or even L<assembly|FFI::Platypus::Lang::ASM>, allowing you to focus 
+on your strengths.
 
 =item FFI / Platypus does not require a parser
 
@@ -823,8 +869,7 @@ prepended to it.
 Returns the list of types that FFI knows about.  This will include the 
 native C<libffi> types (example: C<sint32>, C<opaque> and C<double>) and 
 the normal C types (example: C<unsigned int>, C<uint32_t>), any types 
-that you have defined using the L<type|/type> method, and
-custom types.
+that you have defined using the L<type|/type> method, and custom types.
 
 The list of types that Platypus knows about varies somewhat from 
 platform to platform, L<FFI::Platypus::Type> includes a list of the core 
@@ -843,6 +888,9 @@ Returns a hash reference with the meta information for the given type.
 It can also be called as a class method, in which case, you won't be 
 able to get meta data on user defined types.
 
+The format of the meta data is implementation dependent and subject to 
+change.  It may be useful for display or debugging.
+
 Examples:
 
  my $meta = $ffi->type_meta('int');        # standard int type
@@ -873,13 +921,12 @@ of the symbol yourself:
  my $address = $ffi->find_symbol('my_functon');
  my $function = $ffi->function($address => ...);
 
-Under the covers, L<function|/function> uses 
-L<find_symbol|/find_symbol> when you provide it with a 
-name, but it is useful to keep this in mind as there are alternative 
-ways of obtaining a functions address.  Example: a C function could 
-return the address of another C function that you might want to call, or 
-modules such as L<FFI::TinyCC> produce machine code at runtime that you 
-can call from Platypus.
+Under the covers, L<function|/function> uses L<find_symbol|/find_symbol> 
+when you provide it with a name, but it is useful to keep this in mind 
+as there are alternative ways of obtaining a functions address.  
+Example: a C function could return the address of another C function 
+that you might want to call, or modules such as L<FFI::TinyCC> produce 
+machine code at runtime that you can call from Platypus.
 
 Examples:
 
@@ -896,17 +943,17 @@ Examples:
  $ffi->attach([$address => $perl_name] => \@argument_types => $return_type, sub { ... });
 
 Find and attach a C function as a real live Perl xsub.  The advantage of 
-attaching a function over using the L<function|/function> 
-method is that it is much much much faster since no object resolution 
-needs to be done.  The disadvantage is that it locks the function and 
-the L<FFI::Platypus> instance into memory permanently, since there is no 
-way to deallocate an xsub.
+attaching a function over using the L<function|/function> method is that 
+it is much much much faster since no object resolution needs to be done.  
+The disadvantage is that it locks the function and the L<FFI::Platypus> 
+instance into memory permanently, since there is no way to deallocate an 
+xsub.
 
 If just one I<$name> is given, then the function will be attached in 
 Perl with the same name as it has in C.  The second form allows you to 
 give the Perl function a different name.  You can also provide an 
-address (the third form), just like with the 
-L<function|/function> method.
+address (the third form), just like with the L<function|/function> 
+method.
 
 Examples:
 
@@ -917,10 +964,10 @@ Examples:
 
 [version 0.20]
 
-If the last argument is a code reference, then it will be used as a wrapper
-around the attached xsub.  The first argument to the wrapper will be the
-inner xsub.  This can be used if you need to verify/modify input/output
-data.
+If the last argument is a code reference, then it will be used as a 
+wrapper around the attached xsub.  The first argument to the wrapper 
+will be the inner xsub.  This can be used if you need to verify/modify 
+input/output data.
 
 Examples:
 
@@ -985,18 +1032,26 @@ This function is not very fast, so you might want to save this value as
 a constant, particularly if you need the size in a loop with many 
 iterations.
 
+=head2 alignof
+
+[version 0.21]
+
+ my $align = $ffi->alignof($type);
+
+Returns the alignment of the given type in bytes.
+
 =head2 find_lib
 
 [version 0.20]
 
  $ffi->find_lib( lib => $libname );
 
-This is just a shortcut for calling L<FFI::CheckLib#find_lib> and
-updating the L</lib> attribute appropriately.  Care should be taken
-though, as this method simply passes its arguments to
-L<FFI::CheckLib#find_lib>, so if your module or script is depending
-on a specific feature in L<FFI::CheckLib> then make sure that you
-update your prerequisites appropriately.
+This is just a shortcut for calling L<FFI::CheckLib#find_lib> and 
+updating the L</lib> attribute appropriately.  Care should be taken 
+though, as this method simply passes its arguments to 
+L<FFI::CheckLib#find_lib>, so if your module or script is depending on a 
+specific feature in L<FFI::CheckLib> then make sure that you update your 
+prerequisites appropriately.
 
 =head2 find_symbol
 
@@ -1011,10 +1066,10 @@ Return the address of the given symbol (usually function).
  $ffi->package($package, $file); # usually __PACKAGE__ and __FILE__ can be used
  $ffi->package;                  # autodetect
 
-If you have used L<Module::Build::FFI> to bundle C code with your
-distribution, you can use this method to tell the L<FFI::Platypus> instance
-to look for symbols that came with the dynamic library that was built
-when your distribution was installed.
+If you have used L<Module::Build::FFI> to bundle C code with your 
+distribution, you can use this method to tell the L<FFI::Platypus> 
+instance to look for symbols that came with the dynamic library that was 
+built when your distribution was installed.
 
 =head1 EXAMPLES
 
@@ -1079,15 +1134,15 @@ friendly diagnostic letting the user know that the required library is
 missing, and reduce the number of bogus CPAN testers results that you 
 will get.
 
-Also in this example, we rename some of the functions when they are
+Also in this example, we rename some of the functions when they are 
 placed into Perl space to save typing:
 
  attach [notify_notification_new => 'notify_new']
    => [string,string,string]
    => opaque;
 
-When you specify a list reference as the "name" of the function the
-first element is the symbol name as understood by the dynamic library.
+When you specify a list reference as the "name" of the function the 
+first element is the symbol name as understood by the dynamic library. 
 The second element is the name as it will be placed in Perl space.
 
 Later, when we call C<notify_new>:
@@ -1114,33 +1169,89 @@ functions available from the standard c library and.  Interfaces to
 these and other memory related functions are provided by the 
 L<FFI::Platypus::Memory> module.
 
+=head2 structured data records
+
+ package My::UnixTime;
+ 
+ use FFI::Platypus::Record;
+ 
+ record_layout(qw(
+     int    tm_sec
+     int    tm_min
+     int    tm_hour
+     int    tm_mday
+     int    tm_mon
+     int    tm_year
+     int    tm_wday
+     int    tm_yday
+     int    tm_isdst
+     long   tm_gmtoff
+     string tm_zone
+ ));
+ 
+ my $ffi = FFI::Platypus->new;
+ $ffi->lib(undef);
+ # define a record class My::UnixTime and alias it to "tm"
+ $ffi->type("record(My::UnixTime)" => 'tm');
+ 
+ # attach the C localtime function as a constructor
+ $ffi->attach( localtime => ['time_t*'] => 'tm', sub {
+   my($inner, $class, $time) = @_;
+   $time = time unless defined $time;
+   $inner->(\$time);
+ });
+ 
+ package main;
+ 
+ # now we can actually use our My::UnixTime class
+ my $time = My::UnixTime->localtime;
+ printf "time is %d:%d:%d %s\n",
+   $time->tm_hour,
+   $time->tm_min,
+   $time->tm_sec,
+   $time->tm_zone;
+
+B<Discussion>: C and other machine code languages frequently provide 
+interfaces that include structured data records (known as "structs" in 
+C).  They sometimes provide an API in which you are expected to 
+manipulate these records before and/or after passing them along to C 
+functions.  There are a few ways of dealing with such interfaces, but 
+the easiest way is demonstrated here defines a record class using a 
+specific layout.  For more details see L<FFI::Platypus::Record>. 
+(L<FFI::Platypus::Type> includes some other ways of manipulating 
+structured data records).
+
 =head2 libuuid
 
  use FFI::CheckLib;
- use FFI::Platypus::Declare qw( void opaque string );
+ use FFI::Platypus::Declare
+   'void',
+   [ 'string(37)' => 'uuid_string' ],
+   [ 'record(16)' => 'uuid_t' ];
  use FFI::Platypus::Memory qw( malloc free );
  
  lib find_lib_or_exit lib => 'uuid';
  
- attach uuid_generate => [opaque] => void;
- attach uuid_unparse  => [opaque,opaque] => void;
+ attach uuid_generate => [uuid_t] => void;
+ attach uuid_unparse  => [uuid_t,uuid_string] => void;
  
- my $uuid = malloc sizeof 'char[16]';  # uuid_t
+ my $uuid = "\0" x 16;  # uuid_t
  uuid_generate($uuid);
  
- my $string_opaque = malloc 37;       # 36 bytes to store a UUID string
- uuid_unparse($uuid, $string_opaque);
+ my $string = "\0" x 37; # 36 bytes to store a UUID string 
+                         # + NUL termination
+ uuid_unparse($uuid, $string);
  
- print cast( opaque => string, $string_opaque), "\n";
+ print "$string\n";
 
 B<Discussion>: libuuid is a library used to generate unique identifiers 
 (UUID) for objects that may be accessible beyond the local system.  The 
 library is or was part of the Linux e2fsprogs package.
 
 Knowing the size of objects is sometimes important.  In this example, we 
-use the L<sizeof|/sizeof> function to get the size of 16 
-characters (in this case it is simply 16 bytes).  We also know that the 
-strings "deparsed" by C<uuid_unparse> are exactly 37 bytes.
+use the L<sizeof|/sizeof> function to get the size of 16 characters (in 
+this case it is simply 16 bytes).  We also know that the strings 
+"deparsed" by C<uuid_unparse> are exactly 37 bytes.
 
 =head2 puts and getpid
 
@@ -1231,10 +1342,10 @@ handled seamlessly by Platypus.
 B<Discussion>: Sometimes you will have a pointer to a function from a 
 source other than Platypus that you want to call.  You can use that 
 address instead of a function name for either of the 
-L<function|/function> or L<attach|/attach> 
-methods.  In this example we use L<FFI::TinyCC> to compile a short piece 
-of C code and to give us the address of one of its functions, which we 
-then use to create a perl xsub to call it.
+L<function|/function> or L<attach|/attach> methods.  In this example we 
+use L<FFI::TinyCC> to compile a short piece of C code and to give us the 
+address of one of its functions, which we then use to create a perl xsub 
+to call it.
 
 L<FFI::TinyCC> embeds the Tiny C Compiler (tcc) to provide a 
 just-in-time (JIT) compilation service for FFI.
@@ -1251,7 +1362,6 @@ just-in-time (JIT) compilation service for FFI.
  use FFI::Platypus::Buffer qw( scalar_to_buffer buffer_to_scalar );
  
  my $endpoint = "ipc://zmq-ffi-$$";
- #my $endpoint = "tcp://localhost:3000";
  my $ffi = FFI::Platypus->new;
  
  $ffi->lib(undef); # for puts
@@ -1453,9 +1563,9 @@ way the custom types are implemented.  For more details on custom types
 see L<FFI::Platypus::Type> and L<FFI::Platypus::API>.
 
 Another advanced feature of this example is that we extend the 
-L<FFI::Platypus> class to define our own find_symbol method that prefixes
-the symbol names depending on the class in which they are defined.
-This means we can do this when we define a method for Archive:
+L<FFI::Platypus> class to define our own find_symbol method that 
+prefixes the symbol names depending on the class in which they are 
+defined. This means we can do this when we define a method for Archive:
 
  $ffi->attach( support_filter_all => ['archive'] => 'int' );
 
@@ -1466,8 +1576,8 @@ Rather than this:
    ['archive'] => 'int' );
  );
 
-If you didn't want to create an entire new class just for this little
-trick you could also use something like L<Object::Method> to extend
+If you didn't want to create an entire new class just for this little 
+trick you could also use something like L<Object::Method> to extend 
 C<find_symbol>.
 
 =head2 bzip2
@@ -1529,9 +1639,9 @@ C<find_symbol>.
  my $compressed = compress($original);
  print decompress($compressed, length $original);
 
-B<Discussion>: bzip2 is a compression library.  For simple one shot
-attempts at compression/decompression when you expect the original
-and the result to fit within memory it provides two convenience functions
+B<Discussion>: bzip2 is a compression library.  For simple one shot 
+attempts at compression/decompression when you expect the original and 
+the result to fit within memory it provides two convenience functions 
 C<BZ2_bzBuffToBuffCompress> and C<BZ2_bzBuffToBuffDecompress>.
 
 The first four arguments of both of these C functions are identical, and 
@@ -1659,7 +1769,7 @@ structures like arrays or buffers.  If you prefer not to use C<alloca>
 despite these precautions, then you can turn its use off by setting this 
 environment variable when you run C<Build.PL>:
 
- % env FFI_PLATYPUS_NO_ALLOCA=1 perl Build.PL 
+ % env FFI_PLATYPUS_NO_ALLOCA=1 perl Build.PL
  
  
  NO_ALLOCA:
@@ -1683,6 +1793,11 @@ Declarative interface to Platypus.
 
 Type definitions for Platypus.
 
+=item L<FFI::Platypus::Record>
+
+Define structured data records (C "structs") for use with
+Platypus.
+
 =item L<FFI::Platypus::API>
 
 The custom types API for Platypus.
@@ -1762,6 +1877,19 @@ Another FFI for Perl that doesn't appear to have worked for a long time.
 
 =back
 
+=head1 ACKNOWLEDGMENTS
+
+In addition to the contributors mentioned below, I would like to 
+acknowledge Brock Wilcox (AWWAIID) and Meredith Howard (MHOWARD) whose 
+work on L<FFI::Sweet|https://github.com/merrilymeredith/p5-FFI-Sweet> 
+not only helped me get started with FFI but significantly influenced the 
+design of Platypus.
+
+In addition I'd like to thank Alessandro Ghedini (ALEXBIO) who was 
+always responsive to bug reports and pull requests for L<FFI::Raw>, 
+which was important in the development of the ideas on which Platypus is 
+based.
+
 =cut
 
 =head1 AUTHOR
