@@ -125,7 +125,12 @@ ffi_pl_closure_call(ffi_cif *ffi_cif, void *result, void **arguments, void *user
       {
         sv = sv_newmortal();
         if( *((char**)arguments[i]) != NULL)
-          sv_setpv(sv, *((char**)arguments[i]));
+        {
+          if(extra->argument_types[i]->extra[0].string.platypus_string_type == FFI_PL_STRING_FIXED)
+            sv_setpvn(sv, *((char**)arguments[i]), extra->argument_types[i]->extra[0].string.size);
+          else
+            sv_setpv(sv, *((char**)arguments[i]));
+        }
         XPUSHs(sv);
       }
     }
